@@ -9,13 +9,13 @@ use std::time::Instant;
 #[cfg(not(target_arch = "wasm32"))]
 mod injection;
 
+use im_ternary_tree::TernaryTreeList;
 use notify::{RecommendedWatcher, RecursiveMode, Watcher};
 
 use calcit_runner::{
   builtins, call_stack, cli_args, codegen, codegen::emit_js::gen_stack, codegen::COMPILE_ERRORS_FILE, program, runner, snapshot, util,
   ProgramEntries,
 };
-use im_ternary_tree::TernaryTreeList;
 
 pub struct CLIOptions {
   entry_path: PathBuf,
@@ -73,7 +73,7 @@ fn main() -> Result<(), String> {
 
     let data = cirru_edn::parse(&content)?;
     // println!("reading: {}", content);
-    snapshot = snapshot::load_snapshot_data(data, cli_options.entry_path.to_str().unwrap())?;
+    snapshot = snapshot::load_snapshot_data(&data, cli_options.entry_path.to_str().unwrap())?;
 
     // config in entry will overwrite default configs
     if let Some(entry) = cli_matches.value_of("entry") {
@@ -228,7 +228,7 @@ fn recall_program(content: &str, entries: &ProgramEntries, settings: &CLIOptions
   // 3. rerun program, and catch error
 
   let data = cirru_edn::parse(content)?;
-  let changes = snapshot::load_changes_info(data)?;
+  let changes = snapshot::load_changes_info(&data)?;
 
   // println!("\ndata: {}", &data);
   // println!("\nchanges: {:?}", changes);
