@@ -22,7 +22,7 @@ pub fn parse_cli<'a>() -> clap::ArgMatches<'a> {
     )
     .arg(
       clap::Arg::with_name("emit-ir")
-        .help("emit JSON representation of program to program-ir.json")
+        .help("emit EDN representation of program to program-ir.cirru")
         .default_value("false")
         .long("emit-ir")
         .takes_value(false),
@@ -61,10 +61,9 @@ pub fn parse_cli<'a>() -> clap::ArgMatches<'a> {
         .takes_value(true),
     )
     .arg(
-      clap::Arg::with_name("event-entry")
-        .help("entry ns/def for handling events")
-        .long("event-entry")
-        .default_value("app.main/on-window-event")
+      clap::Arg::with_name("entry")
+        .help("overwrite with config entry")
+        .long("entry")
         .takes_value(true),
     )
     .arg(
