@@ -1,7 +1,9 @@
-use crate::primes::{Calcit, CalcitItems};
+use std::sync::Arc;
+
 use cirru_parser::Cirru;
 use im_ternary_tree::TernaryTreeList;
-use std::sync::Arc;
+
+use crate::primes::Calcit;
 
 /// code is CirruNode, and this function parse code(rather than data)
 pub fn code_to_calcit(xs: &Cirru, ns: Arc<str>, def: Arc<str>) -> Result<Calcit, String> {
@@ -43,67 +45,63 @@ pub fn code_to_calcit(xs: &Cirru, ns: Arc<str>, def: Arc<str>) -> Result<Calcit,
           Ok(n) => Ok(Calcit::Number(n as f64)),
           Err(e) => Err(format!("failed to parse hex: {} => {:?}", s, e)),
         },
-        '\'' if s.len() > 1 => Ok(Calcit::List(
-          TernaryTreeList::Empty
-            .push(Calcit::Symbol {
-              sym: String::from("quote").into(),
-              ns: ns.to_owned(),
-              at_def: def.to_owned(),
-              resolved: None,
-            })
-            .push(Calcit::Symbol {
-              sym: String::from(&s[1..]).into(),
-              ns: ns.to_owned(),
-              at_def: def.to_owned(),
-              resolved: None,
-            }),
-        )),
+        '\'' if s.len() > 1 => Ok(Calcit::List(TernaryTreeList::from(&[
+          Calcit::Symbol {
+            sym: String::from("quote").into(),
+            ns: ns.to_owned(),
+            at_def: def.to_owned(),
+            resolved: None,
+          },
+          Calcit::Symbol {
+            sym: String::from(&s[1..]).into(),
+            ns: ns.to_owned(),
+            at_def: def.to_owned(),
+            resolved: None,
+          },
+        ]))),
         // TODO also detect simple variables
-        '~' if s.starts_with("~@") && s.chars().count() > 2 => Ok(Calcit::List(
-          TernaryTreeList::Empty
-            .push(Calcit::Symbol {
-              sym: String::from("~@").into(),
-              ns: ns.to_owned(),
-              at_def: def.to_owned(),
-              resolved: None,
-            })
-            .push(Calcit::Symbol {
-              sym: String::from(&s[2..]).into(),
-              ns: ns.to_owned(),
-              at_def: def.to_owned(),
-              resolved: None,
-            }),
-        )),
-        '~' if s.chars().count() > 1 && !s.starts_with("~@") => Ok(Calcit::List(
-          TernaryTreeList::Empty
-            .push(Calcit::Symbol {
-              sym: String::from("~").into(),
-              ns: ns.to_owned(),
-              at_def: def.to_owned(),
-              resolved: None,
-            })
-            .push(Calcit::Symbol {
-              sym: String::from(&s[1..]).into(),
-              ns: ns.to_owned(),
-              at_def: def.to_owned(),
-              resolved: None,
-            }),
-        )),
-        '@' => Ok(Calcit::List(
-          TernaryTreeList::Empty
-            .push(Calcit::Symbol {
-              sym: String::from("deref").into(),
-              ns: ns.to_owned(),
-              at_def: def.to_owned(),
-              resolved: None,
-            })
-            .push(Calcit::Symbol {
-              sym: String::from(&s[1..]).into(),
-              ns: ns.to_owned(),
-              at_def: def.to_owned(),
-              resolved: None,
-            }),
-        )),
+        '~' if s.starts_with("~@") && s.chars().count() > 2 => Ok(Calcit::List(TernaryTreeList::from(&[
+          Calcit::Symbol {
+            sym: String::from("~@").into(),
+            ns: ns.to_owned(),
+            at_def: def.to_owned(),
+            resolved: None,
+          },
+          Calcit::Symbol {
+            sym: String::from(&s[2..]).into(),
+            ns: ns.to_owned(),
+            at_def: def.to_owned(),
+            resolved: None,
+          },
+        ]))),
+        '~' if s.chars().count() > 1 && !s.starts_with("~@") => Ok(Calcit::List(TernaryTreeList::from(&[
+          Calcit::Symbol {
+            sym: String::from("~").into(),
+            ns: ns.to_owned(),
+            at_def: def.to_owned(),
+            resolved: None,
+          },
+          Calcit::Symbol {
+            sym: String::from(&s[1..]).into(),
+            ns: ns.to_owned(),
+            at_def: def.to_owned(),
+            resolved: None,
+          },
+        ]))),
+        '@' => Ok(Calcit::List(TernaryTreeList::from(&[
+          Calcit::Symbol {
+            sym: String::from("deref").into(),
+            ns: ns.to_owned(),
+            at_def: def.to_owned(),
+            resolved: None,
+          },
+          Calcit::Symbol {
+            sym: String::from(&s[1..]).into(),
+            ns: ns.to_owned(),
+            at_def: def.to_owned(),
+            resolved: None,
+          },
+        ]))),
         // TODO future work of reader literal expanding
         _ => {
           if let Ok(f) = s.parse::<f64>() {
@@ -120,19 +118,18 @@ pub fn code_to_calcit(xs: &Cirru, ns: Arc<str>, def: Arc<str>) -> Result<Calcit,
       },
     },
     Cirru::List(ys) => {
-      let mut zs: CalcitItems = TernaryTreeList::Empty;
+      let mut zs: Vec<Calcit> = Vec::with_capacity(ys.len());
       for y in ys {
         match code_to_calcit(y, ns.to_owned(), def.to_owned()) {
           Ok(v) => {
             if !is_comment(&v) {
-              zs = zs.push(v.to_owned())
-            } else {
+              zs.push(v.to_owned());
             }
           }
           Err(e) => return Err(e),
         }
       }
-      Ok(Calcit::List(zs))
+      Ok(Calcit::List(TernaryTreeList::from(&zs)))
     }
   }
 }
@@ -142,11 +139,11 @@ pub fn cirru_to_calcit(xs: &Cirru) -> Calcit {
   match xs {
     Cirru::Leaf(s) => Calcit::Str((**s).into()),
     Cirru::List(ys) => {
-      let mut zs: CalcitItems = TernaryTreeList::Empty;
+      let mut zs: Vec<Calcit> = Vec::with_capacity(ys.len());
       for y in ys {
-        zs = zs.push(cirru_to_calcit(y))
+        zs.push(cirru_to_calcit(y));
       }
-      Calcit::List(zs)
+      Calcit::List(TernaryTreeList::from(&zs))
     }
   }
 }
