@@ -1,6 +1,9 @@
 pub mod preprocess;
 pub mod track;
 
+use im_ternary_tree::TernaryTreeList;
+use std::sync::{Arc, RwLock};
+
 use crate::builtins;
 use crate::builtins::is_proc_name;
 use crate::call_stack::{extend_call_stack, CallStackList, StackKind};
@@ -8,9 +11,6 @@ use crate::primes::{Calcit, CalcitErr, CalcitItems, CalcitScope, CalcitSyntax, C
 use crate::program;
 use crate::util::string::has_ns_part;
 
-use im_ternary_tree::TernaryTreeList;
-use std::sync::{Arc, RwLock};
-
 pub fn evaluate_expr(expr: &Calcit, scope: &CalcitScope, file_ns: Arc<str>, call_stack: &CallStackList) -> Result<Calcit, CalcitErr> {
   // println!("eval code: {}", expr.lisp_str());
 
@@ -56,7 +56,7 @@ pub fn evaluate_expr(expr: &Calcit, scope: &CalcitScope, file_ns: Arc<str>, call
         // println!("eval expr: {}", x);
 
         let v = evaluate_expr(x, scope, file_ns.to_owned(), call_stack)?;
-        let rest_nodes = xs.skip(1)?;
+        let rest_nodes = xs.drop_left();
         let ret = match &v {
           Calcit::Proc(p) => {
             let values = evaluate_args(&rest_nodes, scope, file_ns.to_owned(), call_stack)?;
@@ -107,7 +107,7 @@ pub fn evaluate_expr(expr: &Calcit, scope: &CalcitScope, file_ns: Arc<str>, call
               &values,
             );
 
-            run_fn(&values, def_scope, args.to_owned(), body, def_ns.to_owned(), &next_stack)
+            run_fn(&values, def_scope, args, body, def_ns.to_owned(), &next_stack)
           }
           Calcit::Macro {
             name, def_ns, args, body, ..
@@ -118,7 +118,7 @@ pub fn evaluate_expr(expr: &Calcit, scope: &CalcitScope, file_ns: Arc<str>, call
             );
 
             // TODO moving to preprocess
-            let mut current_values = rest_nodes.to_owned();
+            let mut current_values = Box::new(rest_nodes.to_owned());
             // println!("eval macro: {} {}", x, expr.lisp_str()));
             // println!("macro... {} {}", x, CrListWrap(current_values.to_owned()));
 
@@ -133,11 +133,11 @@ pub fn evaluate_expr(expr: &Calcit, scope: &CalcitScope, file_ns: Arc<str>, call
 
             Ok(loop {
               // need to handle recursion
-              let body_scope = bind_args(args.to_owned(), &current_values, &rpds::HashTrieMap::new_sync(), call_stack)?;
+              let body_scope = bind_args(args, &current_values, &rpds::HashTrieMap::new_sync(), call_stack)?;
               let code = evaluate_lines(body, &body_scope, def_ns.to_owned(), &next_stack)?;
               match code {
                 Calcit::Recur(ys) => {
-                  current_values = ys;
+                  current_values = Box::new(ys.to_owned());
                 }
                 _ => {
                   // println!("gen code: {} {}", x, &code.lisp_str()));
@@ -285,18 +285,18 @@ fn eval_symbol_from_program(sym: &str, ns: &str, call_stack: &CallStackList) ->
 pub fn run_fn(
   values: &CalcitItems,
   scope: &CalcitScope,
-  args: Arc<Vec<Arc<str>>>,
+  args: &[Arc<str>],
   body: &CalcitItems,
   file_ns: Arc<str>,
   call_stack: &CallStackList,
 ) -> Result<Calcit, CalcitErr> {
-  let mut current_values = values.to_owned();
+  let mut current_values = Box::new(values.to_owned());
   loop {
-    let body_scope = bind_args(args.to_owned(), &current_values, scope, call_stack)?;
+    let body_scope = bind_args(args, &current_values, scope, call_stack)?;
     let v = evaluate_lines(body, &body_scope, file_ns.to_owned(), call_stack)?;
     match v {
       Calcit::Recur(xs) => {
-        current_values = xs;
+        current_values = Box::new(xs.to_owned());
       }
       result => return Ok(result),
     }
@@ -306,7 +306,7 @@ pub fn run_fn(
 /// create new scope by writing new args
 /// notice that `&` is a mark for spreading, `?` for optional arguments
 pub fn bind_args(
-  args: Arc<Vec<Arc<str>>>,
+  args: &[Arc<str>],
   values: &CalcitItems,
   base_scope: &CalcitScope,
   call_stack: &CallStackList,
@@ -350,7 +350,7 @@ pub fn bind_args(
         _ => {
           let mut chunk: CalcitItems = TernaryTreeList::Empty;
           while let Some(v) = values_pop_front() {
-            chunk = chunk.push(v.to_owned());
+            chunk = chunk.push_right(v.to_owned());
           }
           scope.insert_mut(sym.to_owned(), Calcit::List(chunk));
           if !is_args_empty() {
@@ -422,7 +422,7 @@ pub fn evaluate_args(
   file_ns: Arc<str>,
   call_stack: &CallStackList,
 ) -> Result<CalcitItems, CalcitErr> {
-  let mut ret: CalcitItems = TernaryTreeList::Empty;
+  let mut ret: Vec<Calcit> = Vec::with_capacity(items.len());
   let mut spreading = false;
   for item in items {
     match item {
@@ -444,7 +444,7 @@ pub fn evaluate_args(
                   },
                   _ => x.to_owned(),
                 };
-                ret = ret.push(y.to_owned());
+                ret.push(y.to_owned());
               }
               spreading = false
             }
@@ -464,10 +464,10 @@ pub fn evaluate_args(
             },
             _ => v.to_owned(),
           };
-          ret = ret.push(y);
+          ret.push(y);
         }
       }
     }
   }
-  Ok(ret)
+  Ok(TernaryTreeList::from(&ret))
 }
